@@ -10,7 +10,7 @@
 # one element per categorical parameter.  Each categorical parameter
 # contains a list of vector. This list contains elements which are the
 # .ID. of the configuration. 
-initialiseModel <- function (parameters, configurations)
+initialiseModel <- function (parameters, configurations, digits)
 {
   model <- list()
   nbConfigurations <- nrow(configurations)
@@ -22,15 +22,17 @@ initialiseModel <- function (parameters, configurations)
     if (type == "c") {
       value <- rep((1 / nbValues), nbValues)
     } else if (type == "i" || type == "r") {
-      lowerBound <- paramLowerBound(currentParameter, parameters)
-      upperBound <- paramUpperBound(currentParameter, parameters)
-      value <- (upperBound - lowerBound) / 2
+      value <- init.model.numeric(currentParameter, parameters, type, digits)
     } else {
       irace.assert(type == "o")
       value <- (nbValues - 1) / 2
     }
     for (indexConfig in seq_len(nbConfigurations)) {
       idCurrentConfig <- as.character(configurations[indexConfig, ".ID."])
+      # Assign current parameter value to model
+      if (type == "i" || type == "r") {
+        value[2] <- configurations[indexConfig, currentParameter]
+      }
       param[[idCurrentConfig]] <- value
     }
     model[[currentParameter]] <- param
@@ -51,6 +53,7 @@ updateModel <- function (parameters, eliteConfigurations, oldModel,
 
     for (currentParameter in parameters$names[!parameters$isFixed]) {
       type <- parameters$types[[currentParameter]]
+      transform <- parameters$transform[[currentParameter]]
 
       ## If the elite is older than the current iteration, it has
       ## its own model that has evolved with time. If the elite is
@@ -70,6 +73,10 @@ updateModel <- function (parameters, eliteConfigurations, oldModel,
         idParent <- as.character(idParent)
         # cat("The parent found is ", idParent, "\n")
         probVector <- oldModel[[currentParameter]][[idParent]]
+        #Change the current parameter value of the model
+        if (type %in% c("i", "r") &&
+            !is.na(eliteConfigurations[idCurrentConfiguration,currentParameter]))
+          probVector[2] <- eliteConfigurations[idCurrentConfiguration,currentParameter]
       }
       # cat("probVector: ", probVector)
 
@@ -105,9 +112,14 @@ updateModel <- function (parameters, eliteConfigurations, oldModel,
           #print(newVector)  
         }
       } else {
-        irace.assert(type == "i" || type == "r" || type == "o")
+        irace.assert(type %in% c("i", "r", "o"))
         # Not really a vector but stdDev factor
-        probVector <- probVector * ((1 / nbNewConfigurations)^(1 / parameters$nbVariable))
+        newProbVector <- probVector[1] * ((1 / nbNewConfigurations)^(1 / parameters$nbVariable))
+        if (transform == "log") {
+          probVector[1] <- log(probVector[1] / newProbVector)
+        } else {
+          probVector[1] <- newProbVector
+        }
       }
       newModel[[currentParameter]][[idCurrentConfiguration]] <- probVector
     }
@@ -122,7 +134,7 @@ printModel <- function (model)
 }
 
 restartConfigurations <- function (configurations, restart.ids, model, parameters,
-                               nbConfigurations)
+                               nbConfigurations, digits)
 {
   #print(configurations)
   tmp.ids <- c()
@@ -147,21 +159,51 @@ restartConfigurations <- function (configurations, restart.ids, model, parameter
         probVector <- 0.9 * probVector + 0.1 * max(probVector)
         model[[param]][[id]] <- probVector / sum(probVector)
       } else {
-        irace.assert(type == "i" || type == "r" || type == "o")
         if (type == "i" || type == "r") {
-          lowerBound <- paramLowerBound(param, parameters)
-          upperBound <- paramUpperBound(param, parameters)
-          value <- (upperBound - lowerBound) / 2
+          value <- init.model.numeric(param, parameters, type, digits)
+          # We keep the value of the configuration as last known
+          value[2] <- configurations[id, param]
         } else {
           irace.assert(type == "o")
           value <- (length(parameters$domain[[param]]) - 1) / 2
         }
         # Bring back the value 2 iterations or to the second iteration value.
-        model[[param]][[id]] <-
-          min(model[[param]][[id]] * (nbConfigurations^(2 / parameters$nbVariable)),
-              value * ((1 / nbConfigurations)^(1 / parameters$nbVariable)))
+        model[[param]][[id]][1] <-
+          min(model[[param]][[id]][1] * (nbConfigurations^(2 / parameters$nbVariable)),
+              value[1] * ((1 / nbConfigurations)^(1 / parameters$nbVariable)))
+        if (parameters$transform[[param]] == "log") {
+            model[[param]][[id]][1] <- log(model[[param]][[id]][1])
+        }
       }
     }
   }
   return (model) 
 }
+
+# Initialise model in case of numerical variables.
+# it retuns an array size 2, first number indicates the 
+# standard deviation and second the last known value (initially NA)
+init.model.numeric <- function(param, parameters, type, digits)
+{
+  lowerBound <- paramLowerBound(param, parameters)
+  upperBound <- paramUpperBound(param, parameters)
+  transform <- parameters$transform[[param]]
+  if (transform == "log") {
+    trRange <- range.transform.log(lowerBound, upperBound, digits)
+    trLb <- trRange[["trLowerBound"]]
+    trUb <- trRange[["trUpperBound"]]
+    value <- exp((trLb - trUb) / 2)
+    value <- round(value)
+    value <- check.transform.log(value, lowerBound, upperBound)
+  } else {
+    value <- (upperBound - lowerBound) / 2
+  }
+  if (type == "i") {
+    value <- round(value)
+  } else {
+    value <- round(value, digits)
+  }
+ # 
+  model <- c(value, NA)
+  return(model)
+}
